@@ -58,13 +58,14 @@ class Band:
         n=None,
         M=None,
         high_symm_points=None,
-        bandgap=False,
-        printbg=True,
+        # bandgap=False,
+        # printbg=True,
         shift_efermi=0,
         interpolate=True,
         new_n=200,
         custom_kpath=None,
         soc_axis=None,
+        stretch_factor=1.0,
     ):
         """
         Initialize parameters upon the generation of this class
@@ -89,8 +90,6 @@ class Band:
                 the conver_slab function in the utils module.
             high_symm_points (list[list]): Coordinates of the high symmetry points of the bulk 
                 Brilloin zone for an unfolded calculation.
-            bandgap (bool): Determines is the band gap should be determined or not.
-            printbg (bool): Determines is the band gap value should be printed while plotting
             shift_efermi (float): Gives the option to shift the fermi energy by the specified value
             interpolate (bool): Determines is the data between each high symmetry point should be
                 interpolated or not.
@@ -107,13 +106,16 @@ class Band:
                 and spin='down' states will be defined by negative values of this spin-component.
                 This will only be used for showing a pseudo-spin-polarized plot for calculations
                 that have SOC enabled.
+            stretch_factor (float): Used to scale the eigenvalues by a certain constant. Useful for comparing to ARPES data.
+                Default is scale_factor = 1.0 (i.e. no scaling)
         """
 
         self.interpolate = interpolate
         self.soc_axis = soc_axis
         self.new_n = new_n
-        self.bandgap = bandgap
-        self.printbg = printbg
+        self.stretch_factor = stretch_factor
+        # self.bandgap = bandgap
+        # self.printbg = printbg
         self.eigenval = Eigenval(os.path.join(folder, 'EIGENVAL'))
         self.efermi = float(os.popen(f'grep E-fermi {os.path.join(folder, "OUTCAR")}').read().split()[2]) + shift_efermi
         self.poscar = Poscar.from_file(
@@ -172,6 +174,10 @@ class Band:
         else:
             self.pre_loaded_bands = os.path.isfile(os.path.join(folder, 'unfolded_eigenvalues.npy'))
             self.eigenvalues, self.spectral_weights, self.K_indices, self.kpoints = self._load_bands_unfold()
+
+        if self.stretch_factor != 1.0:
+            self.eigenvalues *= self.stretch_factor
+
         self.color_dict = {
             0: '#FF0000',
             1: '#0000FF',
@@ -214,10 +220,10 @@ class Band:
             'd': 2,
             'f': 3,
         }
-        if self.bandgap:
-            self.bg = self._get_bandgap()
-        else:
-            self.bg = None
+        # if self.bandgap:
+        #     self.bg = self._get_bandgap()
+        # else:
+        #     self.bg = None
 
         self.custom_kpath = custom_kpath
         if self.custom_kpath is not None:
@@ -239,12 +245,12 @@ class Band:
 
         return inds, flip
 
-    def _get_bandgap(self):
-        from vaspvis.utils import get_bandgap
-        self.bg = get_bandgap(
-            folder=self.folder,
-            printbg=self.printbg,
-        )
+    # def _get_bandgap(self):
+    #     from vaspvis.utils import BandGap
+    #     self.bg = BandGap(
+    #         folder=self.folder,
+    #         printbg=self.printbg,
+    #     )
 
     def _check_f_orb(self):
         f_elements = [
@@ -376,6 +382,18 @@ class Band:
         eigenvalues = eigenvalues - self.efermi
         kpath = np.array(kpath)
 
+        path_len = len(self.kpath)
+        n = self.n
+        inserts = [n*(i+1) for i in range(path_len-1)]
+        inds = list(range(n*path_len+1))
+        for i in reversed(inserts):
+            inds.insert(i,i)
+
+        kpath = kpath[inds]
+        spectral_weights = spectral_weights[:, inds]
+        K_indices = K_indices[:, inds]
+        eigenvalues = eigenvalues[:, inds]
+
         return eigenvalues, spectral_weights, K_indices, kpath
 
 
@@ -940,7 +958,8 @@ class Band:
                     merged_label = '|'.join([k[0], k[1]]).replace('$|$', '|')
                     labels.append(merged_label)
 
-        kpoints_index = [0] + [(self.n * i) for i in range(1, len(labels))]
+        n = int(len(self.kpoints) / len(self.kpath))
+        kpoints_index = [0] + [(n * i) for i in range(1, len(labels))]
         kpoints_index[-1] -= 1
 
         for k in kpoints_index:
@@ -1063,8 +1082,9 @@ class Band:
             ]
 
         if unfold and not hse:
+            n = int(len(self.kpoints) / len(self.kpath))
             slices = [
-                slice(i*self.n, (i+1)*self.n, None) for i in range(int(len(self.kpath)))
+                slice(i*n, (i+1)*n, None) for i in range(int(len(self.kpath)))
             ]
 
         return slices
@@ -1101,9 +1121,12 @@ class Band:
             ]
 
         if unfold and not hse:
+            n = int(len(self.kpoints) / len(self.kpath))
+            print(n)
             slices = [
-                slice(i*self.n, (i+1)*self.n, None) for i in range(int(len(self.kpath)-1))
+                slice(i*n, (i+1)*n, None) for i in range(int(len(self.kpath)-1))
             ]
+            print(slices)
 
         return slices
 
@@ -1284,6 +1307,7 @@ class Band:
         slices = self._get_slices(unfold=self.unfold, hse=self.hse)
         wave_vector_segments = self._get_k_distance()
 
+
         if self.soc_axis is not None and self.lsorbit:
             color = 'black'
             linestyle='-'
@@ -1321,6 +1345,7 @@ class Band:
 
             wave_vectors_for_kpoints = wave_vectors
 
+
             if self.interpolate:
                 wave_vectors, eigenvalues = self._get_interpolated_data_segment(
                     wave_vectors_for_kpoints,
@@ -2607,7 +2632,7 @@ if __name__ == "__main__":
         high_symm_points=high_symm_points,
         n=30,
         new_n=50,
-        kpath=[['X', 'G'], ['G', 'X']]
+        kpath=[['X', 'G'], ['G', 'X']],
     )
     band_down = Band(
         #  folder="../../vaspvis_data/band_EuS_bulk/band",
@@ -2627,19 +2652,33 @@ if __name__ == "__main__":
         ax=ax,
         sp_color='red',
         erange=[-6,3],
-        sp_scale_factor=3,
-        scale_factor=10,
+        sp_scale_factor=1,
+        scale_factor=4,
     )
     band_down.plot_plain(
         ax=ax,
         sp_color='blue',
         erange=[-6,3],
-        sp_scale_factor=3,
-        scale_factor=10,
+        sp_scale_factor=1,
+        scale_factor=4,
     )
     ax.set_ylim(-6,3)
     fig.tight_layout()
     fig.savefig('unfold_test.png')
+
+    fig1, ax1 = plt.subplots(figsize=(2.5,4), dpi=400)
+    band_hse = Band(
+        folder='../../vaspvis_data/band_InAs_hse',
+        interpolate=False,
+        projected=True,
+        custom_kpath=[-1,1]
+    )
+    band_hse.plot_spd(
+       ax=ax1, 
+    )
+    ax1.set_ylim([-3,3])
+    fig1.tight_layout(pad=0.4)
+    fig1.savefig('hse.png')
     #  fig.savefig('bulk_test.png')
         
 
