@@ -160,13 +160,18 @@ class Dos:
             self.pdos_array = self._load_pdos()
 
             if self.lsorbit and self.soc_axis is not None:
-                if self.sp_method == 'absolute':
+                if self.spin == 'both':
+                    if self.sp_method == 'absolute':
+                        self.tdos_array[:,1] = self.pdos_array.sum(axis=1).sum(axis=1)
+                    elif self.sp_method == 'percentage':
+                        if self.combination_method == 'add':
+                            raise BaseException("If using combination_method='add' and spin orbit coupling is applied, just set spin='up'")
+                        else:
+                            self.tdos_array[:,1] = (self.pdos_array[0].sum(axis=1).sum(axis=1) - self.pdos_array[1].sum(axis=1).sum(axis=1)) / (self.pdos_array[0].sum(axis=1).sum(axis=1) + self.pdos_array[1].sum(axis=1).sum(axis=1))
+                elif self.spin == 'up':
+                    self.tdos_array[:,1] = self.pdos_array.sum(axis=1).sum(axis=1)
+                elif self.spin == 'down':
                     self.tdos_array[:,1] = self.pdos_array.sum(axis=1).sum(axis=1)
-                elif self.sp_method == 'percentage':
-                    if self.combination_method == 'add':
-                        raise BaseException("If using combination_method='add' and spin orbit coupling is applied, just set spin='up'")
-                    else:
-                        self.tdos_array[:,1] = (self.pdos_array[0].sum(axis=1).sum(axis=1) - self.pdos_array[1].sum(axis=1).sum(axis=1)) / (self.pdos_array[0].sum(axis=1).sum(axis=1) + self.pdos_array[1].sum(axis=1).sum(axis=1))
 
 
     def _check_f_orb(self):
@@ -1006,7 +1011,19 @@ class Dos:
                     )
 
 
-    def plot_plain(self, ax, linewidth=1.5, fill=True, alpha=0.3, alpha_line=1.0, sigma=0.05, energyaxis='y', color='black', erange=[-6, 6], log_scale=False):
+    def plot_plain(
+        self,
+        ax,
+        linewidth=1.5,
+        fill=True,
+        alpha=0.3,
+        alpha_line=1.0,
+        sigma=0.05,
+        energyaxis='y',
+        color='black',
+        erange=[-6, 6],
+        log_scale=False,
+    ):
         """
         This function plots the total density of states
 
@@ -1103,6 +1120,7 @@ class Dos:
         erange=[-6, 6],
         atol=None,
         custom_layer_inds=None,
+        linestyle='-',
     ):
         """
         This function plots the total density of states
@@ -1158,7 +1176,8 @@ class Dos:
                 tdos_array[:,0],
                 linewidth=linewidth,
                 color=color,
-                alpha=alpha_line
+                alpha=alpha_line,
+                linestyle=linestyle,
             )
 
             if fill:
@@ -1176,7 +1195,8 @@ class Dos:
                 tdensity,
                 linewidth=linewidth,
                 color=color,
-                alpha=alpha_line
+                alpha=alpha_line,
+                linestyle=linestyle,
             )
 
             if fill:
@@ -1704,6 +1724,7 @@ class Dos:
         antialiased=False,
         fontsize=6,
         interface_layer=None,
+        show_interface_line=False,
         interface_line_color='white',
         interface_line_width=2,
         interface_line_style='--',
@@ -1736,8 +1757,10 @@ class Dos:
             lrange (list): Upper and lower bounds of the layers included in the plot.
             antialiased (bool): Determines if antialiasing is used or not.
             fontsize (float): Fontsize of all the text in the group.
-            interface_layer (float or None): If a value is provided, then a line will be drawn
-                on the plot to identify the interface layer.
+            interface_layer (float or None): If a value is provided, then the axis labels will be
+                shifted accordingly so that the defined interface layer is zero.
+            show_interface_line (bool): If True, then a line will be drawn
+                on the plot to identify the interface layer defined by interface_layer.
             interface_line_color (str): Color of the line drawn on the plot to mark the 
                 interface.
             interface_line_width (float): Line with of the line marking the interface.
@@ -1837,7 +1860,17 @@ class Dos:
 
                 norm = colors.Normalize(vmin=-norm_val, vmax=norm_val)
             else:
-                norm = colors.Normalize(vmin=np.min(densities), vmax=np.max(densities))
+                if max_cutoff is not None:
+                    max_val = max_cutoff
+                else:
+                    max_val = np.max(densities)
+
+                if min_cutoff is not None:
+                    min_val = min_cutoff
+                else:
+                    min_val = np.min(densities)
+
+                norm = colors.Normalize(vmin=min_val, vmax=max_val)
 
 
         if log_scale:
@@ -1862,6 +1895,9 @@ class Dos:
                     )
             levels = np.power(10, lev_exp)
 
+        if interface_layer is not None:
+            atom_index -= int(interface_layer)
+
         if energyaxis == 'y':
 
             if contour:
@@ -1886,9 +1922,9 @@ class Dos:
             ax.xaxis.set_major_locator(MaxNLocator(integer=True))
 
 
-            if interface_layer is not None:
+            if interface_layer is not None and show_interface_line:
                 ax.axvline(
-                    x=interface_layer,
+                    x=0,
                     color=interface_line_color,
                     linestyle=interface_line_style,
                     linewidth=interface_line_width,
@@ -1916,9 +1952,9 @@ class Dos:
                 )
             ax.yaxis.set_major_locator(MaxNLocator(integer=True))
 
-            if interface_layer is not None:
+            if interface_layer is not None and show_interface_line:
                 ax.axhline(
-                    y=interface_layer,
+                    y=0,
                     color=interface_line_color,
                     linestyle=interface_line_style,
                     linewidth=interface_line_width,
